import unittest
from gfm import *

class TestGitHubFlavoredMarkdown(unittest.TestCase):
    def test_does_not_touch_single_underscores_inside_words(self):
        self.assertEqual("foo_bar", gfm("foo_bar"))

    def test_does_not_touch_underscores_in_code_blocks(self):
        self.assertEqual("    foo_bar_baz", gfm("    foo_bar_baz"))

    def test_does_not_touch_underscores_in_pre_blocks(self):
        self.assertEqual(
            "\n\n<pre>\nfoo_bar_baz\n</pre>",
            gfm("<pre>\nfoo_bar_baz\n</pre>")
        )

    def test_does_not_treat_pre_blocks_with_pre_text_differently(self):
        a = "\n\n<pre>\nthis is `a\\_test` and this\\_too\n</pre>"
        b = "hmm<pre>\nthis is `a\\_test` and this\\_too\n</pre>"
        self.assertEqual(gfm(a)[2:-1], gfm(b)[3:-1])

    def test_does_escape_two_or_more_underscores_inside_words(self):
        self.assertEqual("foo\\_bar\\_baz", gfm("foo_bar_baz"))

    def test_does_turn_newlines_into_br_tags_in_simple_cases(self):
        self.assertEqual("foo  \nbar", gfm("foo\nbar"))

    def test_does_convert_newlines_in_all_groups(self):
        self.assertEqual(
            "apple  \npear  \norange\n\nruby  \npython  \nerlang",
            gfm("apple\npear\norange\n\nruby\npython\nerlang")
        )

    def test_does_convert_newlines_in_long_groups(self):
        self.assertEqual(
            "apple  \npear  \norange  \nbanana\n\nruby  \npython  \nerlang",
            gfm("apple\npear\norange\nbanana\n\nruby\npython\nerlang")
        )

    def test_does_not_convert_newlines_in_lists(self):
        self.assertEqual("# foo\n# bar", gfm("# foo\n# bar"))
        self.assertEqual("* foo\n* bar", gfm("* foo\n* bar"))

if __name__ == "__main__":
    unittest.main()



import hashlib, re
import markdown as markdown_lib

def gfm(text):
    """Processes Markdown according to GitHub Flavored Markdown spec."""
    extractions = {}

    def extract_pre_block(matchobj):
        match = matchobj.group(0)
        hashed_match = hashlib.md5(match.encode('utf-8')).hexdigest()
        extractions[hashed_match] = match
        result = "{gfm-extraction-%s}" % hashed_match
        return result

    def escape_underscore(matchobj):
        match = matchobj.group(0)

        if match.count('_') > 1:
            return re.sub('_', '\_', match)
        else:
            return match

    def newlines_to_brs(matchobj):
        match = matchobj.group(0)
        if re.search("\n{2}", match):
            return match
        else:
            match = match.strip()
            return match + "  \n"

    def insert_pre_block(matchobj):
        string = "\n\n" + extractions[matchobj.group(1)]
        return string

    text = re.sub("<pre>.*?<\/pre>", extract_pre_block, text, flags=re.S)
    text = re.sub("(^(?! {4}|\t)\w+_\w+_\w[\w_]*)", escape_underscore, text)
    text = re.sub("^[\w\<][^\n]*\n+", newlines_to_brs, text, flags=re.M)
    text = re.sub("\{gfm-extraction-([0-9a-f]{32})\}", insert_pre_block, text)

    return text

def markdown(text):
    """Processes GFM then converts it to HTML."""
    text = gfm(text)
    text = markdown_lib.markdown(text)
    return text



from setuptools import setup

setup(
    name='gfm',
    version='0.0.2',
    description='Convert GitHub-Flavored Markdown to HTML',
    long_description=open('README.rst').read(),
    url='http://github.com/stewart/gfm/',
    license=open('LICENSE').read(),
    author='Andrew Stewart',
    author_email='andrew@stwrt.ca',
    py_modules=['gfm'],
    include_package_data=True,
    install_requires=['markdown'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
